"use strict";

function Controller(context, canvasXSize, canvasYSize) {
	this.context = context;
	this.canvasXSize = canvasXSize;
	this.canvasYSize = canvasYSize;
	this.uiButtons = [];
}



Controller.prototype.start = function() {
	this.mode = "title";
	this.titleScreen = new TitleScreen(this);
	//this.play();   // for testing
	this.render();
};



Controller.prototype.render = function() {
	this.context.fillStyle = "#ffffff";
	this.context.fillRect(0, 0, this.canvasXSize, this.canvasYSize);
	switch (this.mode) {
		case "title":
			this.titleScreen.render(this, this.canvasXSize, this.canvasYSize);
			break;
		case "game":
			this.game.render(this, this.canvasXSize, this.canvasYSize);
			break;
	}
	this.renderUI();
};



Controller.prototype.onMouseMove = function(x, y) {
	var changed = false;
	for (var i = 0; i < this.uiButtons.length; i++) {
		var button = this.uiButtons[i];
		var oldHover = button.hover;
		button.hover = x >= button.x0 && x < button.x0 + button.xSize &&
				y >= button.y0 && y < button.y0 + button.ySize;
		if (button.hover !== oldHover)
			changed = true;
	}
	if (changed)
		this.render();
};



Controller.prototype.onMouseDown = function(x, y) {
	var clicked = false;
	for (var i = 0; i < this.uiButtons.length; i++) {
		var button = this.uiButtons[i];
		if (x >= button.x0 && x < button.x0 + button.xSize &&
				y >= button.y0 && y < button.y0 + button.ySize) {
			button.onClick();
			clicked = true;
			break;
		}
	}
	if (clicked)
		this.render();
};



Controller.prototype.play = function() {
	this.uiButtons = [];
	this.mode = "game";
	this.game = new Game(this);
	this.game.start(this);
	this.render();
};



// timer functions


Controller.prototype.startTimer = function() {
	timerObject = this;
	this.intervalID = window.setInterval(timerCallback, 1000);
};



var timerObject;

function timerCallback() {
	timerObject.timerCallback();
}



Controller.prototype.timerCallback = function() {
	this.game.onTimer();
	this.render();
};



Controller.prototype.timerMultiplier = function(multiplier) {
	window.clearInterval(this.intervalID);
	this.intervalID = window.setInterval(timerCallback, 1000 / multiplier);
};



// UI functions

Controller.prototype.uiButton = function(x, xAlign, y, text, fontSize, onClick) {
	this.uiButtons.push({ x: x, xAlign: xAlign, y: y, text: text, fontSize: fontSize,
			onClick: onClick, hover: false });
};



Controller.prototype.renderUI = function() {
	this.context.textBaseline = "top";
	for (var i = 0; i < this.uiButtons.length; i++) {
		var button = this.uiButtons[i];
		this.context.font = button.fontSize + "px Verdana";
		var xSize = this.context.measureText(button.text).width;
		this.context.strokeStyle = button.hover ? "#33CE52" : "#000000";
		button.x0 = button.x - xSize * button.xAlign - 20;
		button.y0 = button.y - 7;
		button.xSize = xSize + 40;
		button.ySize = button.fontSize + 14;
		this.context.strokeRect(button.x0, button.y0, button.xSize, button.ySize);
		this.context.fillStyle = button.hover ? "#196628" : "#000000";
		this.drawText(button.text, button.x, button.y, button.xAlign);
	}
	this.context.textBaseline = "alphabetic";
};



// rendering functions

Controller.prototype.drawText = function(text, x, y, align) {
	var measure = this.context.measureText(text);
	this.context.fillText(text, x - measure.width * align, y);
}